﻿using System;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using Ozeki.Media;
using Ozeki.Media.MediaHandlers;
using Ozeki.Network.Nat;
using Ozeki.VoIP;
using Ozeki.VoIP.Media;
using Ozeki.VoIP.SDK;
using Ozeki.Common;


namespace Voice_Conference_Room
{
    public partial class FormMain : Form
    {
        private ISoftPhone softPhone;
        private IPhoneLine phoneLine;
        private IPhoneCall currentCall;
        bool inComingCall;

        private Microphone microphone;
        private Speaker speaker;
        private ConferenceRoom conferenceRoom;
       

        public FormMain()
        {
            InitializeComponent();
            checkBoxAutoAnswer.Checked = true;
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            statusbarLabelOzeki.Text = string.Format("© Copyright {0}, Ozeki Informatics Ltd.", DateTime.Now.Year);
            InitializeSoftPhone();
        }

        #region Ozeki VoIP-SIP SDK's events

        /// <summary>
        /// Occurs when phone line state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void phoneLine_PhoneLineInformation(object sender, VoIPEventArgs<PhoneLineState> e)
        {
            InvokeGUIThread(() =>
            {
                labelIdentifier.Text = ((IPhoneLine)sender).SIPAccount.RegisterName;
                if (e.Item == PhoneLineState.RegistrationSucceeded)
                {
                    labelRegStatus.Text = "Online";
                    labelCallStateInfo.Text = "Registration succeeded";
                    phoneLine.AutoAnswer = true;
                }
                else
                    labelCallStateInfo.Text = e.Item.ToString();
            });
        }

        /// <summary>
        /// Occurs when an incoming call request has received.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void softPhone_IncomingCall(object sender, VoIPEventArgs<IPhoneCall> e)
        {
            InvokeGUIThread(() =>
            {
                labelCallStateInfo.Text = "Incoming call";
                labelDialingNumber.Text = String.Format("from {0}", e.Item.DialInfo.UserName);
                IPhoneCall call = e.Item;
                inComingCall = true;
                if (phoneLine.AutoAnswer)
                {
                    listBoxActiveCalls.Items.Add(call);
                    WireUpCallEvents(call);
                }
                else
                     currentCall = call;
            });
        }

        /// <summary>
        /// Occurs when the phone call state has changed.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallStateChanged(object sender, VoIPEventArgs<CallState> e)
        {
            InvokeGUIThread(() => { labelCallStateInfo.Text = e.Item.ToString(); });
            IPhoneCall call = sender as IPhoneCall;
            switch (e.Item)
            {
                case CallState.InCall:
                    conferenceRoom.AddToConference(call);
                    break;

                case CallState.Completed:
                    conferenceRoom.RemoveFromConference(call);
                    call = null;

                    InvokeGUIThread(() =>
                    {
                        labelDialingNumber.Text = string.Empty;
                        if (listBoxActiveCalls.Items.Contains(sender))
                        {
                            listBoxActiveCalls.Items.Remove(sender);
                        }
                    });
                    break;

                case CallState.Cancelled:
                    call = null;
                    break;
            }
        }

        /// <summary>
        /// Displays DTMF signals
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_DtmfReceived(object sender, VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal>> e)
        {
            InvokeGUIThread(() =>
            {
                labelCallStateInfo.Text = String.Format("DTMF signal received from '{0}', sign:'{1}'", ((IPhoneCall)sender).PhoneLine.SIPAccount.DisplayName, e.Item.Item2.Signal);
            });
        }

        /// <summary>
        /// There are certain situations when the call cannot be created, for example the dialed number is not available 
        /// or maybe there is no endpoint to the dialed PBX, or simply the telephone line is busy. 
        /// This event handling is for displaying these events.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void call_CallErrorOccured(object sender, VoIPEventArgs<CallError> e)
        {
            InvokeGUIThread(() =>
            {
                labelCallStateInfo.Text = e.Item.ToString();
            });
        }

        #endregion

        #region Keypad events

        /// <summary>
        /// It starts a call with the dialed number or in case of an incoming call it accepts, picks up the call.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonPickUp_Click(object sender, EventArgs e)
        {
            if (inComingCall && currentCall!= null)
            {
                inComingCall = false;
                PickUpCall(currentCall);
                return;
            }

            if (currentCall != null)
                return;
        }

        private void PickUpCall(IPhoneCall p_call)
        {
            WireUpCallEvents(p_call);
            p_call.Accept();
            listBoxActiveCalls.Items.Add(p_call);

            conferenceRoom.AddToConference(p_call);
        }

        /// <summary>
        /// In the case of a call that is in progress it breaks the call.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonHangUp_Click(object sender, EventArgs e)
        {
            if (currentCall != null)
            {
                inComingCall = false;
                currentCall.HangUp();
                currentCall = null;
            }
            labelDialingNumber.Text = string.Empty;
        }

        #endregion


        #region Helper Functions
        /// <summary>
        ///It initializes a softphone object  with a SIP BPX, and it  is for requisiting a SIP account that is nedded  for a SIP PBX service. It registers this SIP
        ///account to the SIP PBX through an ’IphoneLine’ object  which represents the telephoneline. 
        ///If the telephone registration is successful we have a call ready softphone. In this example the softphone can be reached by dialing the number 891.
        /// </summary>
        private void InitializeSoftPhone()
        {
            try
            {
                softPhone = SoftPhoneFactory.CreateSoftPhone(SoftPhoneFactory.GetLocalIP()	, 5700, 5750, 5700);
                softPhone.IncomingCall += new EventHandler<VoIPEventArgs<IPhoneCall>>(softPhone_IncomingCall);
                phoneLine =
                    softPhone.CreatePhoneLine(new SIPAccount(true, "oz875", "oz875", "oz875", "oz875", "192.168.112.100",
                                                             5060),new NatConfiguration(NatTraversalMethod.None,""));
                phoneLine.PhoneLineStateChanged +=
                    new EventHandler<VoIPEventArgs<PhoneLineState>>(phoneLine_PhoneLineInformation);

                softPhone.RegisterPhoneLine(phoneLine);


                microphone =  Microphone.GetDefaultDevice();
                microphone.Start();

                speaker =  Speaker.GetDefaultDevice();
                speaker.Start();
                
                conferenceRoom = new ConferenceRoom();
                conferenceRoom.ConnectSender(microphone);
                conferenceRoom.ConnectReceiver(speaker);
                conferenceRoom.StartConferencing();
            }
            catch (Exception ex)
            {
                MessageBox.Show(String.Format("You didn't give your local IP adress, so the program won't run properly.\n {0}", ex.Message), string.Empty, MessageBoxButtons.OK,
                MessageBoxIcon.Error);
				
                var sb = new StringBuilder();
                sb.AppendLine("Some error happened.");
                sb.AppendLine();
                sb.AppendLine("Exception:");
                sb.AppendLine(ex.Message);
                sb.AppendLine();
                if(ex.InnerException != null)
                {
                    sb.AppendLine("Inner Exception:");
                    sb.AppendLine(ex.InnerException.Message);
                    sb.AppendLine();
                }
                sb.AppendLine("StackTrace:");
                sb.AppendLine(ex.StackTrace);

                MessageBox.Show(sb.ToString());				
            }
        }

        /// <summary>
        ///  It signs up to the necessary events of a call transact.
        /// </summary>
        private void WireUpCallEvents(IPhoneCall p_call)
        {
            p_call.CallStateChanged += new EventHandler<VoIPEventArgs<CallState>>(call_CallStateChanged);
            p_call.DtmfReceived += new EventHandler<VoIPEventArgs<OzTuple<VoIPMediaType, DtmfSignal>>>(call_DtmfReceived);
            p_call.CallErrorOccured += new EventHandler<VoIPEventArgs<CallError>>(call_CallErrorOccured);
        }

        /// <summary>
        /// The controls of the Windows Form applications can only be modified on the GUI thread. This method grants access to the GUI thread.
        /// </summary>
        /// <param name="action"></param>
        private void InvokeGUIThread(Action action)
        {
            Invoke(action);
        }

        private void checkBoxAutoAnswer_CheckedChanged(object sender, EventArgs e)
        {
            if (phoneLine!=null)
                phoneLine.AutoAnswer = checkBoxAutoAnswer.Checked;
            buttonPickUp.Enabled = !checkBoxAutoAnswer.Checked;
        }

        private void listBoxActiveCalls_SelectedValueChanged(object sender, EventArgs e)
        {
            IPhoneCall t = listBoxActiveCalls.SelectedItem as IPhoneCall;
            if (t != null)
                currentCall = t;
        }

        #endregion

        #region Menu events

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Application.Exit();
        }

        private void openOnlineDocumentationToolStripMenuItem_Click(object sender, EventArgs e)
        {
            try
            {
                Process.Start("http://www.voip-sip-sdk.com/");
            }
            catch (Exception exception)
            {
                Debug.WriteLine(exception);
            }
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FormAbout about = new FormAbout();
            about.ShowDialog(this);
        }

        #endregion

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            conferenceRoom.Dispose();
            microphone.Dispose();
            speaker.Dispose();
            softPhone.Close();
        }

    }
}
